/*****
 ForgetMeNot, Door
*****/

#define OUT3 D6   //Yellow wire in the added picture
#define OUT4 D5  //Green wire in the added picture

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
int startTime = 0;

//this is the amount of time the enginge connected to the key is spinning. Measure how long it takes for you to complete a key spin and set it accordingly.
const int timeToMove = 1500;

// Change the credentials below, so your ESP8266 connects to your network. USE A HOTSPOT! Proper WiFi does not work for some reason
const char* ssid = "<WIFI_SSID>";
const char* password = "<WIFI_PASSWORD>";

// Run Mosquitto, do "ifconfig" and get the IP address of your laptop on the HOTSPOT's interface, and add it here
const char* mqtt_server = "<YOUR_LOCAL_MQTT_BROKER_IP>";
 
// Initializes the espClient
WiFiClient espClient;
PubSubClient client(espClient);

// This function connects your ESP8266 to your network
void setup_wifi() {
  delay(10);
  // We start by connecting to a WiFi network
  Serial.begin(9600);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected - ESP IP address: ");
  Serial.println(WiFi.localIP());
}

// This function is executed when some device publishes a message to a topic that your ESP8266 is subscribed to
void callback(String topic, byte* message, unsigned int length) {
  Serial.print("Message arrived on topic: ");
  Serial.print(topic);
  Serial.print(". Message: ");
  String messageTemp;
  for (int i = 0; i < length; i++) {
    Serial.print((char)message[i]);
    messageTemp += (char)message[i];
  }
  Serial.println();

  // If a message is received on the topic house/door, you check if the message is either open or close
  // The door will be locked or unlocked according to the message
  // The serial monitor messages are for you to use when building the system on your own and debugging it
  startTime = millis();
  if(topic=="house/door"){
      Serial.print("Changing door to ");
      if(messageTemp == "closed"){
         startTime = millis();
         while(timeToMove >= millis() - startTime){       
            digitalWrite(OUT3, LOW);
            digitalWrite(OUT4, HIGH);
          }
          Serial.print("closed");
          // Writing LOW to both pins make the motor stop spinning.
          digitalWrite(OUT3, LOW);
          digitalWrite(OUT4, LOW);
      }
      else if(messageTemp == "open"){
        startTime = millis();
       while(timeToMove >= millis() - startTime){
          digitalWrite(OUT3, HIGH);
          digitalWrite(OUT4, LOW);

       }
          Serial.print("open");
          // Writing LOW to both pins make the motor stop spinning.
          digitalWrite(OUT3, LOW);
          digitalWrite(OUT4, LOW);
      }
  }
  Serial.println();
}

// This function reconnects your ESP8266 to your MQTT broker
void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266Door")) {
      Serial.println("connected");  
      client.subscribe("house/door");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

// The setup function sets your ESP pins to outputs, starts the serial communication at a baud rate of 9600, sets your mqtt broker and sets the callback function
// The callback function is what receives messages and actually controls the LEDs
// The pinMode helps you decide if the pin is of type output or input - in this case the type is output - we want the DC motor to spin in reaction to our system's commands
void setup() {
  pinMode(OUT3, OUTPUT);
  pinMode(OUT4, OUTPUT);
  Serial.begin(9600);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void loop() {
  if (!client.connected()) {
    reconnect();
  }
  if(!client.loop()) {
    client.connect("ESP8266Door");
  }
}